local MakePlayerCharacter = require("prefabs/player_common")

local assets =
{
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
    Asset("ANIM", "anim/beard.zip"),
}

local prefabs =
{
    "beardhair",
}

local start_inv =
{
    default =
    {
    },
}
for k, v in pairs(TUNING.GAMEMODE_STARTING_ITEMS) do
	start_inv[string.lower(k)] = v.WILSON
end

prefabs = FlattenTree({ prefabs, start_inv }, true)

local function conversion(inst)

	if inst.components.hunger.current >= (135) and inst.components.hunger.current <= (150) then
	
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 1.25 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 1.25 )
		
	elseif inst.components.hunger.current >= (120) and inst.components.hunger.current < (135) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 1.2 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 1.2 )
		
	elseif inst.components.hunger.current >= (105) and inst.components.hunger.current < (120) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 1.15 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 1.15 )
		
	elseif inst.components.hunger.current >= (90) and inst.components.hunger.current < (105) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 1.1 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 1.1 )
		
	elseif inst.components.hunger.current >= (75) and inst.components.hunger.current < (90) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 1.05 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 1.05 )
		
	elseif inst.components.hunger.current >= (30) and inst.components.hunger.current < (75) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 1.0 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 1.0 )
		
	elseif inst.components.hunger.current >= (15) and inst.components.hunger.current < (30) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 0.95 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 0.95 )
		
	elseif inst.components.hunger.current >= (0) and inst.components.hunger.current < (15) then
	   
		inst.components.locomotor.walkspeed = ( TUNING.WILSON_WALK_SPEED * 0.9 )
		inst.components.locomotor.runspeed = ( TUNING.WILSON_RUN_SPEED * 0.9 )

	end
	
end

local function common_postinit(inst)
    if TheNet:GetServerGameMode() == "quagmire" then
        inst:AddTag("quagmire_foodie")
        inst:AddTag("quagmire_potmaster")
        inst:AddTag("quagmire_shopper")
    end

    --bearded (from beard component) added to pristine state for optimization
    inst:AddTag("bearded")
end

local function OnResetBeard(inst)
    inst.AnimState:ClearOverrideSymbol("beard")
end

--tune the beard economy...
local BEARD_DAYS = { 4, 8, 16 }
local BEARD_BITS = { 1, 3,  9 }

local function OnGrowShortBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "beard", "beard_short")
    inst.components.beard.bits = BEARD_BITS[1]
end

local function OnGrowMediumBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "beard", "beard_medium")
    inst.components.beard.bits = BEARD_BITS[2]
end

local function OnGrowLongBeard(inst)
    inst.AnimState:OverrideSymbol("beard", "beard", "beard_long")
    inst.components.beard.bits = BEARD_BITS[3]
end

local function master_postinit(inst)
    inst.starting_inventory = start_inv[TheNet:GetServerGameMode()] or start_inv.default

    inst:AddComponent("beard")
    inst.components.beard.onreset = OnResetBeard
    inst.components.beard.prize = "beardhair"
    inst.components.beard:AddCallback(BEARD_DAYS[1], OnGrowShortBeard)
    inst.components.beard:AddCallback(BEARD_DAYS[2], OnGrowMediumBeard)
    inst.components.beard:AddCallback(BEARD_DAYS[3], OnGrowLongBeard)
	
	inst:ListenForEvent("hungerdelta", function(inst) conversion(inst) end)

    if TheNet:GetServerGameMode() == "lavaarena" then
        event_server_data("lavaarena", "prefabs/wilson").master_postinit(inst)
    end
end

return MakePlayerCharacter("wilson", prefabs, assets, common_postinit, master_postinit)
